/*
This function is to work around the issue with an array bound to elements of a table.  If we add / remove elements to the array they are not observed by the data source and hence are not kept in sync with the WebView.  This function works around this limitation by cloning the objects of the array that are already observed and then changing the values of the clone to match the values of the passed in array.  
*/
function MIReplaceContentsOfArrayWithObjects(array, replacementArray)
{
    if(!array)
		throw new InvalidArgumentError( "Missing array parameter" );
	if(!replacementArray)
		throw new InvalidArgumentError( "Missing replacementArray parameter" );
	
	var lengthOfArray = array.length;
	var lengthOfReplacementArray = replacementArray.length;
	var limit = MISmallestValue(lengthOfArray, lengthOfReplacementArray);
	var index = 0;
	
	// for each element in the array, change the values to match the passed in array.
	for (; index < limit; index++)
	{
		array[index].key = replacementArray[index]["key"];
		array[index].value = replacementArray[index]["value"];
	}
	
	// If we have more items in the new array than we have in the old one,
	// we'll add the new ones by cloning the first object and changing it's values.
	if (lengthOfArray < lengthOfReplacementArray)
	{
		for (; index < lengthOfReplacementArray; index++)
		{
			// clone an object in the array to keep the registered observers
			array[index] = Object.clone(array[0]);
			
			// set the key and value of the array
			array[index].key = replacementArray[index]["key"];
			array[index].value = replacementArray[index]["value"];
			
			// this following line seems to post notifications of the changed value.
			// warning: this works but we don't know why.
			array.replaceObjectsAtIndexes(new Object(), index);
		}
	}
	
	// If we have less items in the new array we'll delete the rest
	if (lengthOfArray > lengthOfReplacementArray)
	{
		for (index = lengthOfArray; index > replacementArray.length; index--)
		{
			array.removeObjectAtIndex(index-1);
		}
	}

}


function MISetValueForKeyPathInDataSource(value, keyPath, dataSourceName)
{
	var dataSource = dashcode.getDataSource(dataSourceName);
	if (dataSource)
	{
		dataSource.setValueForKeyPath(value, keyPath);
	}

}

function MIValueForKeyPathInDataSource(keyPath, dataSourceName)
{
	var dataSource = dashcode.getDataSource(dataSourceName);
	if (dataSource)
	{
		var isVisible = dataSource.valueForKeyPath(keyPath);
		return isVisible;
	}
}

// This function is a work-in-progress and doesn't function correctly yet.
function createKVOArrayWithObjects(objects)
{
	var array = new Array();
	array.initialiseKeyValueObserving();
	
	var objCount = objects.length;
	
	for (var index = 0; index < objCount; index++)
	{
		var obj = new Object();
		for (key in objects[index])
		{
			obj[key] = objects[index][key];
		}
		
		array.replaceObjectAtIndex(obj, index);
		array.observeElementAtIndex(index);
	}
	return array;	
}


// Returns the largest of the two passed values
function MILargestValue(value1, value2)
{
	if (value1 > value2)
		return value1;
	if (value1 < value2)
		return value2;
	return value1;
}

// Returns the smallest of the two passed values
function MISmallestValue(value1, value2)
{
	if (value1 < value2)
		return value1;
	if (value1 > value2)
		return value2;
	return value1;
}
